module ezfpga(CN1,CN2_38,CN2_40,INT_CLK,VCLK,VTUNE);

// VCO
input VCLK;        // GCK2 VCO Clock
inout VTUNE;       // V-TUNE (to LoopFilter)
// CN1
output [8:7] CN1;
// CN2
input CN2_38;      // GCK1
input CN2_40;      // GCK2
// CN3
input INT_CLK;     // USB-24MHz,or Target(100MHz) or X'talOSC

/////////////////////////////////////////////////////////////////////////////
// GCK clock input I/F
/////////////////////////////////////////////////////////////////////////////

wire x_clk;
IBUFG XCLK_I(.I(INT_CLK),.O(x_clk)); // clock input

wire vco_clk;
IBUFG VCLK_I(.I(VCLK),.O(v_clk)); // clock input

wire cn2_38_in;
IBUFG CLK1(.I(CN2_38),.O(cn2_38_in));

wire cn2_40_in;
IBUFG CLK0(.I(CN2_40),.O(cn2_40_in));

/////////////////////////////////////////////////////////////////////////////
// VCO I/F
/////////////////////////////////////////////////////////////////////////////

wire pll_a , pll_b , vco_reset_n;
wire pll_off = ~pll_a & ~pll_b & vco_reset_n; // GATE
wire pll_level = pll_a & vco_reset_n;             // LEVEL
assign vco_reset_n = 1'b1;

// VCO TUNE
OBUFT_S_2 vtune_o(.O(VTUNE),.I(pll_level),.T(pll_off));

/////////////////////////////////////////////////////////////////////////////
// PLL module I/F
/////////////////////////////////////////////////////////////////////////////

// Divider
reg [13:0] div_a;
reg [15:0] div_b;
wire lock;

pll pll(									 
  .clk_a(x_clk),.div_a(div_a),.clk_b(v_clk),.div_b(div_b),
  .clkd_a(),.clkd_b(),
  .phase_a(pll_a),.phase_b(pll_b),
  .lock(lock)
);

/////////////////////////////////////////////////////////////////////////////
// DLL 2X
/////////////////////////////////////////////////////////////////////////////
wire dll_2x , dll_clk , dll_lock , dll_reset;
BUFG DLL_BUFG(.I(dll_2x),.O(dll_clk));
assign dll_reset = ~lock;

CLKDLL VCLK_DLL(
  .CLKIN(v_clk),
  .CLKFB(dll_clk),
  .CLK2X(dll_2x),
  .LOCKED(dll_lock),
  .RST(dll_reset)
);

wire all_lock=dll_lock & lock;
OBUF LOCK(.O(CN1[7]),.I(all_lock));

/////////////////////////////////////////////////////////////////////////////
// VCO clock divider
/////////////////////////////////////////////////////////////////////////////
reg [23:0] v_cnt;
reg [23:0] v_div;
reg div_clk;

always @(posedge v_clk)
begin
  if(v_cnt==0)
  begin
    v_cnt <= v_div;
    div_clk <= div_clk + 1;
  end else
    v_cnt <= v_cnt - 1;
end

/////////////////////////////////////////////////////////////////////////////
// CLOCK Selecter
/////////////////////////////////////////////////////////////////////////////
reg  [1:0] sel_clk;
wire out_clk=(sel_clk==2'b00) ? v_clk : ((sel_clk==2'b01) ? div_clk : dll_clk);
OBUF OCLK(.O(CN1[8]),.I(out_clk));

/////////////////////////////////////////////////////////////////////////////
// JTAG MEM I/F
/////////////////////////////////////////////////////////////////////////////
wire [2:0] JA;
wire [7:0] JDI;
wire [7:0] JDO;
wire JDE;
wire JCS,JOE,JWE;

jtagmem jtagmem(
  .JA(JA),
  .JD_I(JDI),
  .JD_O(JDO),
  .JD_E(JDE),
  .JCS(JCS),
  .JOE(JOE),
  .JWE(JWE)
);

/////////////////////////////////////////////////////////////////////////////
// HOST I/F
/////////////////////////////////////////////////////////////////////////////
always @(posedge JWE)
begin
  case(JA[2:0])
  3'b000:div_a[ 7: 0] <= JDO;
  3'b001:div_a[13: 8] <= JDO[5:0];
  3'b010:div_b[ 7: 0] <= JDO;
  3'b011:div_b[15: 8] <= JDO[7:0];
  3'b100:v_div[ 7: 0] <= JDO;
  3'b101:v_div[15: 8] <= JDO;
  3'b110:v_div[23:16] <= JDO;
  3'b111:sel_clk      <= JDO[1:0];
 endcase
end

assign JDI[0]=lock;
assign JDI[1]=dll_lock;

endmodule
